/*
 * $Id: StubResolver.java 292 2006-02-28 23:30:47Z davidb $
 *
 * Copyright (c) 2004 USC Project SRO. All rights reserved.
 * Copyright (c) 2005 VeriSign, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package se.rfc.unbound.stub;

import java.io.IOException;
import java.util.*;

import org.apache.log4j.Logger;
import org.xbill.DNS.*;

import se.rfc.unbound.*;
import se.rfc.unbound.AbstractResolver;
import se.rfc.unbound.DNSEvent;
import se.rfc.unbound.Host;
import se.rfc.unbound.Request;
import se.rfc.unbound.Resolver;
import se.rfc.unbound.SMessage;

/**
 * This is a resolver module that implements the "stub" resolver logic. The
 * core of this logic is so simple, it almost seems like this module shouldn't
 * exist. However, this module is generally responsible for targeting a
 * request to one of a set of known targets. Since it is responsible for
 * reading /etc/resolv.conf (or the equivalent), it also provides a way to get
 * to the search list, even though it is not this modules' responsibility to
 * *apply* the search list.
 * 
 * @author davidb
 * @version $Revision: 292 $
 */
public class StubResolver extends AbstractResolver
{
  private static class StubEventState
  {
    public Iterator targetListIterator;
  }

  private List   targetList = null;
  private List   searchList = null;

  private Logger log        = Logger.getLogger(this.getClass());

  public StubResolver(Resolver nextResolver)
  {
    super(nextResolver);

    targetList = new ArrayList();
    searchList = new ArrayList();
  }

  private void parseTargetList(String s) throws IOException
  {
    String[] servs = s.split("\\s+");
    for (int i = 0; i < servs.length; i++)
    {
      if (servs[i].indexOf(':') > 0)
      {
        String[] servels = servs[i].split(":", 1);
        targetList.add(new Host(servels[0], servels[1]));
      }
      else
      {
        targetList.add(new Host(servs[i]));
      }
    }

  }

  private void parseSearchPath(String s)
  {
    String[] paths = s.split("\\s+");

    for (int i = 0; i < paths.length; i++)
    {
      try
      {
        Name n = Name.fromString(paths[i], Name.root);
        searchList.add(n);
      }
      catch (TextParseException e)
      {}
    }
  }

  private void fetchOSTargets() throws IOException
  {
    String[] servers = ResolverConfig.getCurrentConfig().servers();

    for (int i = 0; i < servers.length; i++)
    {
      targetList.add(new Host(servers[i]));
    }
  }

  private void fetchOSSearchPath()
  {
    Name[] paths = ResolverConfig.getCurrentConfig().searchPath();

    for (int i = 0; i < paths.length; i++)
    {
      searchList.add(paths[i]);
    }
  }

  public void init(Properties config) throws Exception
  {
    super.init(config);

    String s = null;

    if (config != null) s = config.getProperty("dns.servers");
    if (s != null)
    {
      parseTargetList(s);
    }
    else
    {
      // If not overridden by the configuration, get config information from
      // /etc/resolv.conf or equivalent.
      fetchOSTargets();
    }

    if (config != null) s = config.getProperty("dns.search");
    if (s != null)
    {
      parseSearchPath(s);
    }
    else
    {
      fetchOSSearchPath();
    }
  }

  public void processRequest(DNSEvent event)
  {
    log.trace("processing request");
    
    Request req = event.getRequest();

    // Make sure the RD bit is set.
    req.setFlag(Flags.RD);

    if (targetList.size() == 0)
    {
      log.error("No nameservers configured");
      SMessage err = Util.errorMessage(req, Rcode.SERVFAIL);
      event.setResponse(err);
      processResponse(event);
      return;
    }

    StubEventState state = (StubEventState) event.getModuleState(this);
    if (state == null)
    {
      state = new StubEventState();
      event.setModuleState(this, state);
    }

    if (state.targetListIterator == null)
    {
      state.targetListIterator = targetList.iterator();
    }

    if (!state.targetListIterator.hasNext())
    {
      return;
    }

    Host nst = (Host) state.targetListIterator.next();
    req.setTarget(nst);

    mNextResolver.processRequest(event);
  }

  public void processResponse(DNSEvent event)
  {
    log.trace("processing response");
    
    mPrevResolver.processResponse(event);
  }

  public List getSearchPath()
  {
    return searchList;
  }

}